﻿param([string]$InitDesc)
$global:UserNames = @{};
$global:Desc = "";

function getPlatform($desc)
{
    $platform = "";
    if($desc -like "*SEH*" -or $desc -like "*exchange*")
    {
        $platform = "SEH";
    }
    elseif($desc -like "*DEX*")
    {
        $platform = "DEX";
    }
    else
    {
        $platform = "PLR";
    }

    return $platform;
}

function getEnvironment($desc)
{
    $env = "";
    if($desc -like "*QA*")
    {
		$env = "QA";
    } 
    elseif($desc -like "*DEV*")
    {
		$env = "DEV";
    }

    return $env;
}

function getPlatformName($desc)
{
    $platform = getPlatform $desc;
    $env = getEnvironment $desc;

	if($env)
    {
    	$platform = $env+" "+$platform;
    }
	return $platform;
}

function getPlatformUrl($desc)
{
    $powerShellUrl = "";
    if($desc)
	{
	    $platform = getPlatform $desc;
	    $env = getEnvironment $desc;

	    if($platform -contains "SEH")
	    {
	        $powerShellUrl = "https://exchange.{0}intermedia.net/powershell";
	    }
	    elseif($platform -contains "DEX")
	    {
	        $powerShellUrl = "https://dex.{0}intermedia.net/powershell";
	    }
	    else
	    {
	        $powerShellUrl = "https://cp.{0}serverdata.net/powershell";   
	    }

	    $powerShellUrl = [string]::Format($powerShellUrl,$env);
	}
	
	return $powerShellUrl;
}

function getPlatformColor($platform)
{
    $platformColor = "";
    if($platform)
	{
	    if($platform -contains "SEH")
	    {
	        $platformColor = "Cyan";
	    }
	    elseif($platform -contains "DEX")
	    {
	        $platformColor = "DarkYellow";
	    }
	    else
	    {
	        $platformColor = "Magenta";
	    }
    }
    return $platformColor;
}

function prompt 
{ 
    $platform = "NOT CONNECTED";
	$platformColor = "Red";
    if($global:desc) 
	{
        $platform = getPlatform $global:desc;
        $platformColor = getPlatformColor $platform;

        $platform = getPlatformName $global:desc;
	}

    write-host "[" -ForegroundColor "Yellow" -NoNewLine;
    write-host $platform -ForegroundColor $platformColor -NoNewLine;
    write-host "]" $(get-location) -ForegroundColor "Yellow" -NoNewLine;
    write-host ">" -ForegroundColor "Green" -NoNewLine;
    if($platform -eq "PLR") 
    { 
		$title = "Control Panel PowerShell";
	} 
	else
	{
		$title = "HostPilot PowerShell: $platform";
	}

    $host.UI.RawUI.WindowTitle = $title;
    $host.UI.RawUI.BackgroundColor = "black"
    $host.UI.RawUI.ForegroundColor = "gray"

    return " "
}

function WidenWindow([int]$preferredWidth, [int]$preferredHeight)
{
	[int]$maxAllowedWindowWidth = $host.ui.rawui.MaxPhysicalWindowSize.Width
	if ($preferredWidth -lt $maxAllowedWindowWidth)
	{
		# first, buffer size has to be set to windowsize or more
		# this operation does not usually fail
		$current=$host.ui.rawui.BufferSize
		$bufferWidth = $current.width
		if ($bufferWidth -lt $preferredWidth)
		{
			$current.width=$preferredWidth
			$host.ui.rawui.BufferSize=$current
		}
		# else not setting BufferSize as it is already larger

		# setting window size. As we are well within max limit, it won't throw exception.
		$current = $host.ui.rawui.WindowSize
		if ($current.width -lt $preferredWidth)
		{
		  $current.width=$preferredWidth
		  $current.height=$preferredHeight
		  $host.ui.rawui.WindowSize=$current
		}
	#else not setting WindowSize as it is already larger
	}
}

<#
  Section: PSCredmanUtils
  Author : Jim Harrison (jim@isatools.org)
  Date   : 2012/05/20
#>
[String] $PsCredmanUtils = @"
using System;
using System.Text;
using System.Runtime.InteropServices;
using System.Management.Automation;
using System.Security.Permissions;
using System.Security;

namespace PsUtils
{
    public class CredMan
    {
		[DllImport("credui", EntryPoint = "CredUIPromptForCredentialsW", CharSet = CharSet.Unicode)]
		private static extern CredUIReturnCodes CredUIPromptForCredentials(ref CREDUI_INFO pUiInfo, string pszTargetName, IntPtr Reserved, int dwAuthError, StringBuilder pszUserName, int ulUserNameMaxChars, StringBuilder pszPassword, int ulPasswordMaxChars, ref int pfSave, CREDUI_FLAGS dwFlags);

		[DllImport("credui", EntryPoint = "CredUIConfirmCredentialsW", CharSet = CharSet.Unicode)]
		private static extern CredUIReturnCodes CredUIConfirmCredentials(string targetName, bool confirm);
  
		[Flags]
		private enum CREDUI_FLAGS
		{
			ALWAYS_SHOW_UI = 0x80,
			COMPLETE_USERNAME = 0x800,
			DO_NOT_PERSIST = 2,
			EXCLUDE_CERTIFICATES = 8,
			EXPECT_CONFIRMATION = 0x20000,
			GENERIC_CREDENTIALS = 0x40000,
			INCORRECT_PASSWORD = 1,
			KEEP_USERNAME = 0x100000,
			PASSWORD_ONLY_OK = 0x200,
			PERSIST = 0x1000,
			REQUEST_ADMINISTRATOR = 4,
			REQUIRE_CERTIFICATE = 0x10,
			REQUIRE_SMARTCARD = 0x100,
			SERVER_CREDENTIAL = 0x4000,
			SHOW_SAVE_CHECK_BOX = 0x40,
			CREDUI_FLAGS_EXPECT_CONFIRMATION = 0x20000,
			USERNAME_TARGET_CREDENTIALS = 0x80000,
			VALIDATE_USERNAME = 0x400
		}

		[StructLayout(LayoutKind.Sequential)]
		private struct CREDUI_INFO
		{
			public int cbSize;
			public IntPtr hwndParent;
			[MarshalAs(UnmanagedType.LPWStr)]
			public string pszMessageText;
			[MarshalAs(UnmanagedType.LPWStr)]
			public string pszCaptionText;
			public IntPtr hbmBanner;
		}

		private enum CredUIReturnCodes
		{
			ERROR_CANCELLED = 0x4c7,
			ERROR_INSUFFICIENT_BUFFER = 0x7a,
			ERROR_INVALID_ACCOUNT_NAME = 0x523,
			ERROR_INVALID_FLAGS = 0x3ec,
			ERROR_INVALID_PARAMETER = 0x57,
			ERROR_NO_SUCH_LOGON_SESSION = 0x520,
			ERROR_NOT_FOUND = 0x490,
			NO_ERROR = 0
		}

		public static void ConfirmCredential(string targetName, bool confirm)
		{
			CredUIReturnCodes codes = CredUIConfirmCredentials(targetName, confirm);
			if (codes != CredUIReturnCodes.NO_ERROR)
			{
				throw new ApplicationException("CredUIConfirmCredentials returned an error: " + codes.ToString());
			}
		}
		
		public static PSCredential PromptForCredential(string caption, string message, string userName, string targetName, PSCredentialTypes allowedCredentialTypes, PSCredentialUIOptions options, ref bool save)
		{
			PSCredential credential = null;
			if (string.IsNullOrEmpty(caption))
			{
				caption = "Credential Request";
			}
			if (string.IsNullOrEmpty(message))
			{
				message = "Enter your credentials.";
			}
			CREDUI_INFO structure = new CREDUI_INFO();
			structure.pszCaptionText = caption;
			structure.pszMessageText = message;
			StringBuilder pszUserName = new StringBuilder(userName, 0x201);
			StringBuilder pszPassword = new StringBuilder(0x100);
			int pfSave = Convert.ToInt32(save);
			structure.cbSize = Marshal.SizeOf(structure);
			structure.hwndParent = IntPtr.Zero;
			CREDUI_FLAGS dwFlags = CREDUI_FLAGS.SHOW_SAVE_CHECK_BOX;
			if ((allowedCredentialTypes & PSCredentialTypes.Domain) != PSCredentialTypes.Domain)
			{
				dwFlags |= CREDUI_FLAGS.GENERIC_CREDENTIALS;
				if ((options & PSCredentialUIOptions.AlwaysPrompt) == PSCredentialUIOptions.AlwaysPrompt)
				{
					dwFlags |= CREDUI_FLAGS.ALWAYS_SHOW_UI;
				}
			}
			CredUIReturnCodes codes = CredUIReturnCodes.ERROR_INVALID_PARAMETER;
			if ((pszUserName.Length <= 0x201) && (pszPassword.Length <= 0x100))
			{
				codes = CredUIPromptForCredentials(ref structure, targetName, IntPtr.Zero, 0, pszUserName, 0x201, pszPassword, 0x100, ref pfSave, dwFlags);
				save = Convert.ToBoolean(pfSave);
			}
			if (codes == CredUIReturnCodes.NO_ERROR)
			{
				string str = null;
				if (pszUserName != null)
				{
					str = pszUserName.ToString();
				}
				SecureString password = new SecureString();
				for (int i = 0; i < pszPassword.Length; i++)
				{
					password.AppendChar(pszPassword[i]);
					pszPassword[i] = '\0';
				}
				if (!string.IsNullOrEmpty(str))
				{
					credential = new PSCredential(str, password);
				}
				else
				{
					credential = null;
				}
			}
			else
			{
				if (codes != CredUIReturnCodes.ERROR_CANCELLED)
				{
					throw new OperationCanceledException("CredUIPromptForCredentials returned an error: " + codes.ToString());
				}
				credential = null;
			}
			return credential;
		}
				
        #region Imports
        // DllImport derives from System.Runtime.InteropServices
        [DllImport("Advapi32.dll", SetLastError = true, EntryPoint = "CredDeleteW", CharSet = CharSet.Unicode)]
        private static extern bool CredDeleteW([In] string target, [In] CRED_TYPE type, [In] int reservedFlag);

        [DllImport("Advapi32.dll", SetLastError = true, EntryPoint = "CredEnumerateW", CharSet = CharSet.Unicode)]
        private static extern bool CredEnumerateW([In] string Filter, [In] int Flags, out int Count, out IntPtr CredentialPtr);

        [DllImport("Advapi32.dll", SetLastError = true, EntryPoint = "CredFree")]
        private static extern void CredFree([In] IntPtr cred);

        [DllImport("Advapi32.dll", SetLastError = true, EntryPoint = "CredReadW", CharSet = CharSet.Unicode)]
        private static extern bool CredReadW([In] string target, [In] CRED_TYPE type, [In] int reservedFlag, out IntPtr CredentialPtr);

        [DllImport("Advapi32.dll", SetLastError = true, EntryPoint = "CredWriteW", CharSet = CharSet.Unicode)]
        private static extern bool CredWriteW([In] ref Credential userCredential, [In] UInt32 flags);
        #endregion

        #region Fields
        public enum CRED_ERRORS : uint
        {
            ERROR_SUCCESS = 0x0,
            ERROR_INVALID_PARAMETER = 0x80070057,
            ERROR_INVALID_FLAGS = 0x800703EC,
            ERROR_NOT_FOUND = 0x80070490,
            ERROR_NO_SUCH_LOGON_SESSION = 0x80070520,
            ERROR_BAD_USERNAME = 0x8007089A
        }

        public enum CRED_PERSIST : uint
        {
            SESSION = 1,
            LOCAL_MACHINE = 2,
            ENTERPRISE = 3
        }

        public enum CRED_TYPE : uint
        {
            GENERIC = 1,
            DOMAIN_PASSWORD = 2,
            DOMAIN_CERTIFICATE = 3,
            DOMAIN_VISIBLE_PASSWORD = 4,
            GENERIC_CERTIFICATE = 5,
            DOMAIN_EXTENDED = 6,
            MAXIMUM = 7,      // Maximum supported cred type
            MAXIMUM_EX = (MAXIMUM + 1000),  // Allow new applications to run on old OSes
        }

        [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Unicode)]
        public struct Credential
        {
            public UInt32 Flags;
            public CRED_TYPE Type;
            public string TargetName;
            public string Comment;
            public DateTime LastWritten;
            public UInt32 CredentialBlobSize;
            public string CredentialBlob;
            public CRED_PERSIST Persist;
            public UInt32 AttributeCount;
            public IntPtr Attributes;
            public string TargetAlias;
            public string UserName;
        }

        [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Unicode)]
        private struct NativeCredential
        {
            public UInt32 Flags;
            public CRED_TYPE Type;
            public IntPtr TargetName;
            public IntPtr Comment;
            public System.Runtime.InteropServices.ComTypes.FILETIME LastWritten;
            public UInt32 CredentialBlobSize;
            public IntPtr CredentialBlob;
            public UInt32 Persist;
            public UInt32 AttributeCount;
            public IntPtr Attributes;
            public IntPtr TargetAlias;
            public IntPtr UserName;
        }
        #endregion

        #region Child Class
        private class CriticalCredentialHandle : Microsoft.Win32.SafeHandles.CriticalHandleZeroOrMinusOneIsInvalid
        {
            public CriticalCredentialHandle(IntPtr preexistingHandle)
            {
                SetHandle(preexistingHandle);
            }

            private Credential XlateNativeCred(IntPtr pCred)
            {
                NativeCredential ncred = (NativeCredential)Marshal.PtrToStructure(pCred, typeof(NativeCredential));
                Credential cred = new Credential();
                cred.Type = ncred.Type;
                cred.Flags = ncred.Flags;
                cred.Persist = (CRED_PERSIST)ncred.Persist;

                long LastWritten = ncred.LastWritten.dwHighDateTime;
                LastWritten = (LastWritten << 32) + ncred.LastWritten.dwLowDateTime;
                cred.LastWritten = DateTime.FromFileTime(LastWritten);

                cred.UserName = Marshal.PtrToStringUni(ncred.UserName);
                cred.TargetName = Marshal.PtrToStringUni(ncred.TargetName);
                cred.TargetAlias = Marshal.PtrToStringUni(ncred.TargetAlias);
                cred.Comment = Marshal.PtrToStringUni(ncred.Comment);
                cred.CredentialBlobSize = ncred.CredentialBlobSize;
                if (0 < ncred.CredentialBlobSize)
                {
                    cred.CredentialBlob = Marshal.PtrToStringUni(ncred.CredentialBlob, (int)ncred.CredentialBlobSize / 2);
                }
                return cred;
            }

            public Credential GetCredential()
            {
                if (IsInvalid)
                {
                    throw new InvalidOperationException("Invalid CriticalHandle!");
                }
                Credential cred = XlateNativeCred(handle);
                return cred;
            }

            public Credential[] GetCredentials(int count)
            {
                if (IsInvalid)
                {
                    throw new InvalidOperationException("Invalid CriticalHandle!");
                }
                Credential[] Credentials = new Credential[count];
                IntPtr pTemp = IntPtr.Zero;
                for (int inx = 0; inx < count; inx++)
                {
                    pTemp = Marshal.ReadIntPtr(handle, inx * IntPtr.Size);
                    Credential cred = XlateNativeCred(pTemp);
                    Credentials[inx] = cred;
                }
                return Credentials;
            }

            override protected bool ReleaseHandle()
            {
                if (IsInvalid)
                {
                    return false;
                }
                CredFree(handle);
                SetHandleAsInvalid();
                return true;
            }
        }
        #endregion

        #region Custom API
        public static int CredDelete(string target, CRED_TYPE type)
        {
            if (!CredDeleteW(target, type, 0))
            {
                return Marshal.GetHRForLastWin32Error();
            }
            return 0;
        }

        public static int CredEnum(string Filter, out Credential[] Credentials)
        {
            int count = 0;
            int Flags = 0x0;
            if (string.IsNullOrEmpty(Filter) ||
                "*" == Filter)
            {
                Filter = null;
                if (6 <= Environment.OSVersion.Version.Major)
                {
                    Flags = 0x1; //CRED_ENUMERATE_ALL_CREDENTIALS; only valid is OS >= Vista
                }
            }
            IntPtr pCredentials = IntPtr.Zero;
            if (!CredEnumerateW(Filter, Flags, out count, out pCredentials))
            {
                Credentials = null;
                return Marshal.GetHRForLastWin32Error(); 
            }
            CriticalCredentialHandle CredHandle = new CriticalCredentialHandle(pCredentials);
            Credentials = CredHandle.GetCredentials(count);
            return 0;
        }

        public static int CredRead(string target, CRED_TYPE type, out Credential Credential)
        {
            IntPtr pCredential = IntPtr.Zero;
            Credential = new Credential();
            if (!CredReadW(target, type, 0, out pCredential))
            {
                return Marshal.GetHRForLastWin32Error();
            }
            CriticalCredentialHandle CredHandle = new CriticalCredentialHandle(pCredential);
            Credential = CredHandle.GetCredential();
            return 0;
        }

        public static int CredWrite(Credential userCredential)
        {
            if (!CredWriteW(ref userCredential, 0))
            {
                return Marshal.GetHRForLastWin32Error();
            }
            return 0;
        }

        #endregion

        private static int AddCred()
        {
            Credential Cred = new Credential();
            string Password = "Password";
            Cred.Flags = 0;
            Cred.Type = CRED_TYPE.GENERIC;
            Cred.TargetName = "Target";
            Cred.UserName = "UserName";
            Cred.AttributeCount = 0;
            Cred.Persist = CRED_PERSIST.ENTERPRISE;
            Cred.CredentialBlobSize = (uint)Password.Length;
            Cred.CredentialBlob = Password;
            Cred.Comment = "Comment";
            return CredWrite(Cred);
        }

        private static bool CheckError(string TestName, CRED_ERRORS Rtn)
        {
            switch(Rtn)
            {
                case CRED_ERRORS.ERROR_SUCCESS:
                    Console.WriteLine(string.Format("'{0}' worked", TestName));
                    return true;
                case CRED_ERRORS.ERROR_INVALID_FLAGS:
                case CRED_ERRORS.ERROR_INVALID_PARAMETER:
                case CRED_ERRORS.ERROR_NO_SUCH_LOGON_SESSION:
                case CRED_ERRORS.ERROR_NOT_FOUND:
                case CRED_ERRORS.ERROR_BAD_USERNAME:
                    Console.WriteLine(string.Format("'{0}' failed; {1}.", TestName, Rtn));
                    break;
                default:
                    Console.WriteLine(string.Format("'{0}' failed; 0x{1}.", TestName, Rtn.ToString("X")));
                    break;
            }
            return false;
        }

        /*
         * Note: the Main() function is primarily for debugging and testing in a Visual 
         * Studio session.  Although it will work from PowerShell, it's not very useful.
         */
        public static void Main()
        {
            Credential[] Creds = null;
            Credential Cred = new Credential();
            int Rtn = 0;

            Console.WriteLine("Testing CredWrite()");
            Rtn = AddCred();
            if (!CheckError("CredWrite", (CRED_ERRORS)Rtn))
            {
                return;
            }
            Console.WriteLine("Testing CredEnum()");
            Rtn = CredEnum(null, out Creds);
            if (!CheckError("CredEnum", (CRED_ERRORS)Rtn))
            {
                return;
            }
            Console.WriteLine("Testing CredRead()");
            Rtn = CredRead("Target", CRED_TYPE.GENERIC, out Cred);
            if (!CheckError("CredRead", (CRED_ERRORS)Rtn))
            {
                return;
            }
            Console.WriteLine("Testing CredDelete()");
            Rtn = CredDelete("Target", CRED_TYPE.GENERIC);
            if (!CheckError("CredDelete", (CRED_ERRORS)Rtn))
            {
                return;
            }
            Console.WriteLine("Testing CredRead() again");
            Rtn = CredRead("Target", CRED_TYPE.GENERIC, out Cred);
            if (!CheckError("CredRead", (CRED_ERRORS)Rtn))
            {
                Console.WriteLine("if the error is 'ERROR_NOT_FOUND', this result is OK.");
            }
        }
    }
}
"@

$PsCredMan = $null
try
{
	$PsCredMan = [PsUtils.CredMan];
}
catch
{
	$Error.RemoveAt($Error.Count - 1);
}
if ($null -eq $PsCredMan) { Add-Type $PsCredmanUtils; }

[HashTable] $ErrorCategory = @{0x80070057 = "InvalidArgument";
                               0x800703EC = "InvalidData";
                               0x80070490 = "ObjectNotFound";
                               0x80070520 = "SecurityError";
                               0x8007089A = "SecurityError"}

function Remove-Creds
{
	Param([Parameter(Mandatory=$true)][String] $Target);
    [Int] $Results = 0;
	try
	{
		$Results = [PsUtils.CredMan]::CredDelete($Target, [PsUtils.CredMan+CRED_TYPE]::GENERIC);
	}
	catch
	{
		return $_;
	}
	switch ($Results)
	{
        0 { break }
        0x80070490 { break } #ERROR_NOT_FOUND
        default
        {
    		[String] $Msg = "Failed to delete from target '$Target' credentials store for '$Env:UserName'"
    		[Management.ManagementException] $MgmtException = New-Object Management.ManagementException($Msg)
    		[Management.Automation.ErrorRecord] $ErrRcd = New-Object Management.Automation.ErrorRecord($MgmtException, $Results.ToString("X"), $ErrorCategory[$Results], $null)
    		return $ErrRcd
        }
	}
	return $Cred
}
							   
function Read-Creds
{
	Param([Parameter(Mandatory=$true)][AllowEmptyString()][String] $Target)
	[PsUtils.CredMan+Credential] $Cred = New-Object PsUtils.CredMan+Credential
    [Int] $Results = 0
	try
	{
		$Results = [PsUtils.CredMan]::CredRead($Target, [PsUtils.CredMan+CRED_TYPE]::GENERIC, [ref]$Cred)
	}
	catch
	{
		return $_
	}
	switch($Results)
	{
        0 { break }
        0x80070490 { break } #ERROR_NOT_FOUND
        default
        {
    		[String] $Msg = "Failed to read from target '$Target' credentials store for '$Env:UserName'"
    		[Management.ManagementException] $MgmtException = New-Object Management.ManagementException($Msg)
    		[Management.Automation.ErrorRecord] $ErrRcd = New-Object Management.Automation.ErrorRecord($MgmtException, $Results.ToString("X"), $ErrorCategory[$Results], $null)
    		return $ErrRcd
        }
	}
	return $Cred
}

function Write-Creds
{
	Param(
		[Parameter(Mandatory=$true)][AllowEmptyString()][String] $Target,
		[Parameter(Mandatory=$true)][String] $UserName,
		[Parameter(Mandatory=$true)][String] $Password,
		[Parameter(Mandatory=$false)][String] $Comment = [String]::Empty,
		[Parameter(Mandatory=$false)][Boolean] $EntStrg = $false
	)
	if ([String]::IsNullOrEmpty($Target))
	{
		$Target = $UserName
	}
    if ([String]::IsNullOrEmpty($Comment) -or 256 -lt $Comment.Length)
	{
        $Comment = [String]::Format("Last edited by {0}\{1} on {2}",
                                    $Env:UserDomain,
                                    $Env:UserName,
                                    $Env:ComputerName)
    }
	[Int] $Persist = [PsUtils.CredMan+CRED_PERSIST]::LOCAL_MACHINE
	[String] $DomainName = [Net.NetworkInformation.IPGlobalProperties]::GetIPGlobalProperties().DomainName
	if(-not [String]::IsNullOrEmpty($DomainName) -or ($EntStrg -and -not [String]::IsNullOrEmpty($DomainName))){
		$Persist = [PsUtils.CredMan+CRED_PERSIST]::ENTERPRISE
	}
	[PsUtils.CredMan+Credential] $Cred = New-Object PsUtils.CredMan+Credential
	$Cred.Flags = 0
	$Cred.Type = [PsUtils.CredMan+CRED_TYPE]::GENERIC
	$Cred.TargetName = $Target
	$Cred.UserName = $UserName
	$Cred.AttributeCount = 0
	$Cred.Persist = $Persist
	$Cred.CredentialBlobSize = [Text.Encoding]::Unicode.GetBytes($Password).Length
	$Cred.CredentialBlob = $Password
	$Cred.Comment = $Comment
	[Int] $Results = 0
	try
	{
		$Results = [PsUtils.CredMan]::CredWrite($Cred)
	}
	catch
	{
		return $_
	}
	if (0 -ne $Results)
	{
		[String] $Msg = "Failed to write to credentials store for target '$Target' using '$UserName', '$Password', '$Comment'"
		[Management.ManagementException] $MgmtException = New-Object Management.ManagementException($Msg)
		[Management.Automation.ErrorRecord] $ErrRcd = New-Object Management.Automation.ErrorRecord($MgmtException, $Results.ToString("X"), $ErrorCategory[$Results], $null)
		return $ErrRcd
	}
	return $Results
}

function GetHPCred([string]$plat, [string]$credentialType, [bool]$IgnoreExisting)
{	
	$URL = getPlatformUrl $plat
	$target = ([String]::Format("{0}.{1}",$credentialType,$URL));	
	if ((CheckHPCred -plat $plat -credentialType $credentialType ) -and !$IgnoreExisting)
	{
		$credObj = Read-Creds -Target $target
		$password = $credObj.CredentialBlob | ConvertTo-SecureString -asPlainText -Force;
		$cred = New-Object System.Management.Automation.PsCredential($credObj.UserName,$password);
		Remove-Variable -Name credObj,pwd -Force -ErrorAction SilentlyContinue -WarningAction SilentlyContinue; #contains plaintext PW
	}
	else
	{
		$success = $false
		while (!$success)
		{
			try 
			{
				$save = $false;
				$cred = [PsUtils.CredMan]::PromptForCredential("Please enter credentials", "Please enter your credentials.", [Environment]::UserName, $target, "Generic", "None", [ref] $save);
				if ($cred)
				{
					$success = Test-HPCredential -cred $cred -plat $plat;
					if(-not $success)
					{							
						Remove-Creds -Target $target;
						$cred = $null;
					}				
				}
				else
				{
					Write-Host "Operation cancelled.";
					exit;
				}
			}
			catch 
			{
				exit; 
			}
		}
		if ($save)
		{
			Write-Host "Saving credentials ...";
			Set-SavedHPCredential -cred $cred -plat $plat -credentialType $credentialType;
			Write-Host "To clear saved credentials use Clear-SavedHPCredential command.";
		}
	}
	return $cred
}

function CheckHPCred([string]$plat, [string]$credentialType)
{
	$URL = getPlatformUrl $plat
    $target = [String]::Format("{0}.{1}", $credentialType, $URL);
	$credObj = Read-Creds -Target $target
	if (([string]$credObj.Username).length -gt 0)
	{
		return $true
	}
	else
	{
		return $false
	}
}

function Set-SavedHPCredential([string]$plat, [string]$credentialType, [Management.Automation.PSCredential]$cred)
{
	if(-not $cred)
	{
		throw "Credential required.";
	}
	$URL = getPlatformUrl $plat;
    $target = [String]::Format("{0}.{1}",$credentialType,$URL);
	Write-Host -ForegroundColor DarkGray "Saving credentials ..." -NoNewline;
	$pw = [Runtime.InteropServices.Marshal]::PtrToStringAuto([Runtime.InteropServices.Marshal]::SecureStringToBSTR($cred.Password));
	Write-Creds -Target $target -UserName $cred.UserName -Password $pw | Out-Null;
	Remove-Variable -Name pw -Force -ErrorAction SilentlyContinue -WarningAction SilentlyContinue; #contains plaintext pw
	Write-Host -ForegroundColor DarkGray " success.";
}

function Clear-SavedHPCredential
{
	$platform = getPlatform $global:desc;
	$URL = getPlatformUrl $platform;
    $target = [String]::Format("{0}.{1}",$global:CredentialType,$URL); 

	Write-Host -ForegroundColor DarkGray "Removing saved credentials ..." -NoNewline;
	Remove-Creds -Target $target;
	Remove-Variable -Name pw -Force -ErrorAction SilentlyContinue -WarningAction SilentlyContinue; #contains plaintext pw
	Write-Host -ForegroundColor DarkGray " success.";
}

function Test-HPCredential([string]$plat, [Management.Automation.PSCredential]$cred)
{
	$URL = getPlatformUrl $plat;
	Write-Host -ForegroundColor DarkGray "Testing credentials ..." -NoNewline;
	try
	{
		New-PSSession -name:$plat -ConnectionUri $URL -Credential $cred -Authentication Basic -ConfigurationName IntermediaCredTest -ErrorAction Stop | Remove-PSSession;
		throw "Condition not allowed. Please check the code."
	}
	catch
	{
		if ($_.ErrorDetails.Message -like "*IntermediaCredTest*")
		{
			Write-Host -ForegroundColor DarkGray " success."
			return $true
		}
		else
		{
			Write-Host -ForegroundColor Red " failed."
			return $false
		}
	}
}

function ConnectHP([string]$plat, [Management.Automation.PSCredential]$cred)
{
	$URL = getPlatformUrl $plat;
	Write-Host -ForegroundColor DarkGray "Trying $(getPlatformURL($plat)) ..." -NoNewline;
	$pso = New-PSSessionOption -OperationTimeout (1800*1000);
	$session = New-PSSession -Name:$plat -ConfigurationName Hosting.PowerShell -Authentication Basic -ConnectionUri $URL -Credential $cred -SessionOption $pso;
	if ($session)
	{
		Write-Host -ForegroundColor DarkGray " success.";
		return $session;
	}
	else
	{
		Write-Host -ForegroundColor Red " failed.";
		return $false;
	}
}

function connect($desc, $showCommands, $credType)
{    
    if (-not $desc)
    {
        $desc = $InitDesc;
    }
	
    if (-not $desc)
    {
        $desc = Read-Host "Enter a platform name (seh, dex, plr, ...)";
    } 
	
	$choices = New-Object Collections.ObjectModel.Collection[Management.Automation.Host.ChoiceDescription]
	$choices.Add((New-Object Management.Automation.Host.ChoiceDescription -ArgumentList '&Yes'))
	$choices.Add((New-Object Management.Automation.Host.ChoiceDescription -ArgumentList '&No'))
	$platform = getPlatform $desc;
	
	if (-not $credType)
	{ 
		$credType =$InitCredType;
	}
	
    if (-not $credType)
    {
        $admin = New-Object System.Management.Automation.Host.ChoiceDescription "&Administrator", `
		"Administrator mode.";
        $user = New-Object System.Management.Automation.Host.ChoiceDescription "&User", `
		"User mode.";
        $options = [System.Management.Automation.Host.ChoiceDescription[]]($admin, $user);
		if($platform -eq "PLR") 
	    { 
			write-host "-----------------------------------------------------------------`n"
			write-host "`t Welcome to Control Panel PowerShell`n"
			write-host "-----------------------------------------------------------------"
			write-host "For help please refer to Partner FAQ `n"
			$result = $host.ui.PromptForChoice("", "Please use ‘Administrator’ credential type to login to your Partner Account", $options, 0);  			
		}
		else
		{
			write-host "-----------------------------------------------------------------`n"
			write-host "`t Welcome to Intermedia HostPilot PowerShell`n"
			write-host "-----------------------------------------------------------------"
			write-host "For help use kb article:  https://kb.intermedia.net/Article/23283 `n"
			$result = $host.ui.PromptForChoice("", "Please use ‘User’ credential type to login your account", $options, 1);
		}
        
        switch ($result)
        {
            0 {$credType="Administrator"}
            1 {$credType="User"}
        }                      
    }	
	
    $powerShellUrl = getPlatformUrl $desc;
    $platformName = getPlatformName $desc;
	
    try
    {
	    $existingSession = Get-PSSession | where {$_.Name -eq $platformName };
		if( $global:CredentialType -ne $credType -and $existingSession)
		{
			close $platformName;
			$existingSession = $null;
		}
		
	    $foundSession = $existingSession;

	    if (-not $existingSession)
	    {
			$creds = GetHPCred -plat $platformName -credentialType $credType -IgnoreExisting $true
			# Because all output is captured, and returned. The return keyword in function just indicates a logical exit point.
			if($creds.GetType().FullName -eq 'System.Object[]' )
			{
				$creds = $creds[-1]
			}
			$existingSession = ConnectHP -plat $platformName -cred $creds
	    }

        if ($existingSession)
		{
		    $importedSession = Import-PSSession $existingSession -AllowClobber;        
		    if (-not $foundSession)
	    	{
				if ($showCommands) 
				{
					$importedSession.ExportedCommands.Values | Select-Object Name;
				}
				
				if($credType -eq "User")
				{
					$accountID = Read-Host 'Please enter your accountID'
					Set-ConnectionSettings -Credential $creds -CredentialType $credType -AccountID $accountID;
				}
				else
				{
					Set-ConnectionSettings -Credential $creds -CredentialType $credType;
				}
			
            	$global:UserNames[$platformName] = $creds.UserName;
	    	}

		    Write-Host "Connection to $platformName was successful." -ForegroundColor Yellow
			$global:CredentialType = $credType;			
		    $global:desc = $platformName;
		}
    } 
	catch 
	{
		#Process-Exception $_.Exception 100
	    $global:desc = "";
	    Write-Warning "Cannot connect to $platformName : $($_.Exception.Message)";
	}
}

function close($desc)
{
    if ($desc)
    {
        $platformName = getPlatformName $desc;
        Get-PSSession | where { $_.Name -eq $platformName } | Remove-PSSession;
		if ($global:desc -eq $platformName)
		{
		    $global:desc = "";
		}
    }
    else
    {
        Get-PSSession | Remove-PSSession;
		$global:desc = "";
    }
}

function Process-Exception { param ([Exception]$exception, [int]$eventID)

    $dateTime = Get-Date
	write-host ("EventID:`t" + $eventID)
    write-host ("Exception Source:`t" + $exception.Source)
    write-host ("Error Code:`t"+ $exception.NativeErrorCode)
    write-host ("Exception Message:" + $exception.Message)
    write-host ("Stack Trace:`t" + $exception.StackTrace + "`r`n`r`n")
}

WidenWindow 120 50

set-alias list       format-list 
set-alias table      format-table
del alias:gcm -Force
del alias:measure -Force
Set-ExecutionPolicy -Scope Process -ExecutionPolicy Unrestricted -Force;

[Net.ServicePointManager]::SecurityProtocol = [Net.SecurityProtocolType]::Tls12
connect $InitDesc $false $InitCredType;
# SIG # Begin signature block
# MIINUQYJKoZIhvcNAQcCoIINQjCCDT4CAQExDzANBglghkgBZQMEAgEFADB5Bgor
# BgEEAYI3AgEEoGswaTA0BgorBgEEAYI3AgEeMCYCAwEAAAQQH8w7YFlLCE63JNLG
# KX7zUQIBAAIBAAIBAAIBAAIBADAxMA0GCWCGSAFlAwQCAQUABCARLzhh3hw9LlHq
# ODwVCGEX1vDh4LPM67AQKqGWTLnXNaCCCm4wggUwMIIEGKADAgECAhAECRgbX9W7
# ZnVTQ7VvlVAIMA0GCSqGSIb3DQEBCwUAMGUxCzAJBgNVBAYTAlVTMRUwEwYDVQQK
# EwxEaWdpQ2VydCBJbmMxGTAXBgNVBAsTEHd3dy5kaWdpY2VydC5jb20xJDAiBgNV
# BAMTG0RpZ2lDZXJ0IEFzc3VyZWQgSUQgUm9vdCBDQTAeFw0xMzEwMjIxMjAwMDBa
# Fw0yODEwMjIxMjAwMDBaMHIxCzAJBgNVBAYTAlVTMRUwEwYDVQQKEwxEaWdpQ2Vy
# dCBJbmMxGTAXBgNVBAsTEHd3dy5kaWdpY2VydC5jb20xMTAvBgNVBAMTKERpZ2lD
# ZXJ0IFNIQTIgQXNzdXJlZCBJRCBDb2RlIFNpZ25pbmcgQ0EwggEiMA0GCSqGSIb3
# DQEBAQUAA4IBDwAwggEKAoIBAQD407Mcfw4Rr2d3B9MLMUkZz9D7RZmxOttE9X/l
# qJ3bMtdx6nadBS63j/qSQ8Cl+YnUNxnXtqrwnIal2CWsDnkoOn7p0WfTxvspJ8fT
# eyOU5JEjlpB3gvmhhCNmElQzUHSxKCa7JGnCwlLyFGeKiUXULaGj6YgsIJWuHEqH
# CN8M9eJNYBi+qsSyrnAxZjNxPqxwoqvOf+l8y5Kh5TsxHM/q8grkV7tKtel05iv+
# bMt+dDk2DZDv5LVOpKnqagqrhPOsZ061xPeM0SAlI+sIZD5SlsHyDxL0xY4PwaLo
# LFH3c7y9hbFig3NBggfkOItqcyDQD2RzPJ6fpjOp/RnfJZPRAgMBAAGjggHNMIIB
# yTASBgNVHRMBAf8ECDAGAQH/AgEAMA4GA1UdDwEB/wQEAwIBhjATBgNVHSUEDDAK
# BggrBgEFBQcDAzB5BggrBgEFBQcBAQRtMGswJAYIKwYBBQUHMAGGGGh0dHA6Ly9v
# Y3NwLmRpZ2ljZXJ0LmNvbTBDBggrBgEFBQcwAoY3aHR0cDovL2NhY2VydHMuZGln
# aWNlcnQuY29tL0RpZ2lDZXJ0QXNzdXJlZElEUm9vdENBLmNydDCBgQYDVR0fBHow
# eDA6oDigNoY0aHR0cDovL2NybDQuZGlnaWNlcnQuY29tL0RpZ2lDZXJ0QXNzdXJl
# ZElEUm9vdENBLmNybDA6oDigNoY0aHR0cDovL2NybDMuZGlnaWNlcnQuY29tL0Rp
# Z2lDZXJ0QXNzdXJlZElEUm9vdENBLmNybDBPBgNVHSAESDBGMDgGCmCGSAGG/WwA
# AgQwKjAoBggrBgEFBQcCARYcaHR0cHM6Ly93d3cuZGlnaWNlcnQuY29tL0NQUzAK
# BghghkgBhv1sAzAdBgNVHQ4EFgQUWsS5eyoKo6XqcQPAYPkt9mV1DlgwHwYDVR0j
# BBgwFoAUReuir/SSy4IxLVGLp6chnfNtyA8wDQYJKoZIhvcNAQELBQADggEBAD7s
# DVoks/Mi0RXILHwlKXaoHV0cLToaxO8wYdd+C2D9wz0PxK+L/e8q3yBVN7Dh9tGS
# dQ9RtG6ljlriXiSBThCk7j9xjmMOE0ut119EefM2FAaK95xGTlz/kLEbBw6RFfu6
# r7VRwo0kriTGxycqoSkoGjpxKAI8LpGjwCUR4pwUR6F6aGivm6dcIFzZcbEMj7uo
# +MUSaJ/PQMtARKUT8OZkDCUIQjKyNookAv4vcn4c10lFluhZHen6dGRrsutmQ9qz
# sIzV6Q3d9gEgzpkxYz0IGhizgZtPxpMQBvwHgfqL2vmCSfdibqFT+hKUGIUukpHq
# aGxEMrJmoecYpJpkUe8wggU2MIIEHqADAgECAhAGCMzOz1OAKUDwCLINKl39MA0G
# CSqGSIb3DQEBCwUAMHIxCzAJBgNVBAYTAlVTMRUwEwYDVQQKEwxEaWdpQ2VydCBJ
# bmMxGTAXBgNVBAsTEHd3dy5kaWdpY2VydC5jb20xMTAvBgNVBAMTKERpZ2lDZXJ0
# IFNIQTIgQXNzdXJlZCBJRCBDb2RlIFNpZ25pbmcgQ0EwHhcNMjEwMzExMDAwMDAw
# WhcNMjQwMzE1MjM1OTU5WjB0MQswCQYDVQQGEwJVUzETMBEGA1UECBMKQ2FsaWZv
# cm5pYTESMBAGA1UEBxMJU3Vubnl2YWxlMR0wGwYDVQQKExRJbnRlcm1lZGlhLk5l
# dCwgSW5jLjEdMBsGA1UEAxMUSW50ZXJtZWRpYS5OZXQsIEluYy4wggEiMA0GCSqG
# SIb3DQEBAQUAA4IBDwAwggEKAoIBAQCyMN5DAD3OeX+WpmJBWsYWTLzUPw1L6DKr
# SR2Cy8D8gsUKn8D+wPYMeJmrzZGuZZMI9RQfpsTK2IEQPavth2CcV8EeKgMJ7FXP
# 2o9dE7w/9Ung4bfcO1y4iJey3uZweS7xZ5XVbdt5upAU2Jb3oV+oT3Tia15xacm0
# cwPcSRTG04HLvBjL0/+7uwACK6pDL71dV0238dcJTBt92HmwQVcpyJxJ7fS7qg0G
# sI8N4Q4bX2ZJ/YzMNDmGcFGqLY3wWUxp4gRsdLBUZSh0K0q+xQq1l1Dz7aVmTJqT
# 2YUaLFeQ9nNfUPtBaNmEZ0oXiTc4RKSQm5/0jFcV5OIKCSChVonPAgMBAAGjggHE
# MIIBwDAfBgNVHSMEGDAWgBRaxLl7KgqjpepxA8Bg+S32ZXUOWDAdBgNVHQ4EFgQU
# W1nKOgaHl/E2RGq3q7IeWg4ojoowDgYDVR0PAQH/BAQDAgeAMBMGA1UdJQQMMAoG
# CCsGAQUFBwMDMHcGA1UdHwRwMG4wNaAzoDGGL2h0dHA6Ly9jcmwzLmRpZ2ljZXJ0
# LmNvbS9zaGEyLWFzc3VyZWQtY3MtZzEuY3JsMDWgM6Axhi9odHRwOi8vY3JsNC5k
# aWdpY2VydC5jb20vc2hhMi1hc3N1cmVkLWNzLWcxLmNybDBLBgNVHSAERDBCMDYG
# CWCGSAGG/WwDATApMCcGCCsGAQUFBwIBFhtodHRwOi8vd3d3LmRpZ2ljZXJ0LmNv
# bS9DUFMwCAYGZ4EMAQQBMIGEBggrBgEFBQcBAQR4MHYwJAYIKwYBBQUHMAGGGGh0
# dHA6Ly9vY3NwLmRpZ2ljZXJ0LmNvbTBOBggrBgEFBQcwAoZCaHR0cDovL2NhY2Vy
# dHMuZGlnaWNlcnQuY29tL0RpZ2lDZXJ0U0hBMkFzc3VyZWRJRENvZGVTaWduaW5n
# Q0EuY3J0MAwGA1UdEwEB/wQCMAAwDQYJKoZIhvcNAQELBQADggEBAHacEmC5FWkV
# ev72Y2ElldLD8dEX7MxsQG9w3AvLUV0hHPVaAAW1HXfdtIHmt3JFV0gpVAle73gk
# rSljHGVvSHyCdoKWLptvrKmTvXO38c2v7cf/pRtVNcI0ojxn2IkD5RiR7jP6rt4x
# 34G2o0Oypxqvd1W/O2udYGGEpFFyD71tXhsJGJMB+3enhwfWJSqKYqpG/miVGqZ7
# FDzx4RimjMKL+rzzs3tPHqEj7/dkwXCx+uYM/KMkWGnnvWSE9PuOXNiiK4kIr34c
# mraH+fd6+iib/Eb4H/65VQwtqqPdeh7OkxJq5y+ZJPznDbuF/j9DcwoMRAlsXEf/
# y8KrLyQdip0xggI5MIICNQIBATCBhjByMQswCQYDVQQGEwJVUzEVMBMGA1UEChMM
# RGlnaUNlcnQgSW5jMRkwFwYDVQQLExB3d3cuZGlnaWNlcnQuY29tMTEwLwYDVQQD
# EyhEaWdpQ2VydCBTSEEyIEFzc3VyZWQgSUQgQ29kZSBTaWduaW5nIENBAhAGCMzO
# z1OAKUDwCLINKl39MA0GCWCGSAFlAwQCAQUAoIGEMBgGCisGAQQBgjcCAQwxCjAI
# oAKAAKECgAAwGQYJKoZIhvcNAQkDMQwGCisGAQQBgjcCAQQwHAYKKwYBBAGCNwIB
# CzEOMAwGCisGAQQBgjcCARUwLwYJKoZIhvcNAQkEMSIEINsWwMsWtfzyJVkyrIrk
# 7SwFq6K6FqufSMGFumCFnGb/MA0GCSqGSIb3DQEBAQUABIIBACDi1GN4JoeV2t0V
# LvemCDcfU3YDNekGJ5CMkOgpuf3sKpaGy8fSs9r4CEoY7E7PyuKXpStOCAggggY9
# ZsOtcgmbs3NOF8f15/KL0Lrw+JlFclif2+3ZJ0qkyutzdyR45EZoE2UqghITTs6i
# O/xZu/2mguvXwcRSdshvf1sjABxR4475y0TYjLyj66bUTedOnvzMa0B+Ym+SQrWV
# YAkB2d23H21ONKZqKKA+Mjt6waDz1o8LkGGXiinGMXtFXD/3HbaneyhSJAaze6V1
# XUE/AKI6dJOxnZV7iQA52fgLzRcgJ5Goq7QwY6dSdLRP+S6W5NsCAWLca38uDIEu
# 1ADej48=
# SIG # End signature block
